"""API runner base class.
API runners for example : Public API Runner, Authenticated API Runner, inherit from the base class
to access the different features.
This module also has classes for authentication errors, API response errors, etc"""

import abc
from typing import Dict
import requests

from ostorlab import configuration_manager as config_manager
from ostorlab.apis import request as api_request


class Error(Exception):
    """Base Error Class"""


class ResponseError(Error):
    """Response Error."""


class APIRunner(abc.ABC):
    """Handles API calls and behind the scenes operations."""

    def __init__(self,
                 proxy: str = None,
                 verify: bool = True):
        """Constructs the necessary attributes for the object.

        Args:
            proxy: The proxy through which a request is made. Defaults to None.
            verify: Whether or not to verify the TLS certificate. Defaults to True.
        """
        self._proxy = proxy
        self._verify = verify
        self._configuration_manager: config_manager.ConfigurationManager = config_manager.ConfigurationManager()


    @property
    def endpoint(self) -> str:
        """API endpoint."""
        raise NotImplementedError('Missing implementation')

    @abc.abstractmethod
    def execute(self, request: api_request.APIRequest) -> Dict:
        """Executes a request using the GraphQL API.

        Args:
            request: The request to be executed

        Raises:
            ResponseError: When the API returns an error

        Returns:
            The API response
        """
        raise NotImplementedError('Missing implementation')

    def _sent_request(self, request: api_request.APIRequest, headers=None) -> requests.Response:
        """Sends an API request."""
        if self._proxy is not None:
            proxy = {
                'https': self._proxy
            }
        else:
            proxy = None

        return requests.post(self.endpoint, data=request.data, files=request.files, headers=headers,
                             proxies=proxy, verify=self._verify)



"""Asset of type IP.
This module takes care of preparing an IP asset, either single address, a range with mask for both v4 and v6."""
import ipaddress
import logging

import click

from ostorlab.assets import ipv4
from ostorlab.assets import ipv6
from ostorlab.cli import console as cli_console
from ostorlab.cli.scan.run import run

logger = logging.getLogger(__name__)

console = cli_console.Console()


@run.run.command(name='ip')
@click.argument('ips', required=True, nargs=-1)
@click.pass_context
def ip_cli(ctx: click.core.Context, ips: str) -> None:
    """Run scan for IP asset."""
    runtime = ctx.obj['runtime']
    try:
        assets = []
        for ip in ips:
            ip_network = ipaddress.ip_network(ip, strict=False)
            if ip_network.version == 4:
                assets.append(ipv4.IPv4(host=ip_network.network_address.exploded, mask=str(ip_network.prefixlen)))
            elif ip_network.version == 6:
                assets.append(ipv6.IPv6(host=ip_network.network_address.exploded, mask=str(ip_network.prefixlen)))
            else:
                console.error(f'Invalid Ip address {ip}')

        logger.debug('scanning assets %s', assets)
        runtime.scan(title=ctx.obj['title'], agent_group_definition=ctx.obj['agent_group_definition'], assets=assets)
    except ValueError as e:
        console.error(f'{e}')
        click.Abort()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v3/asset/link/link.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='v3/asset/link/link.proto',
  package='v3.asset.link',
  syntax='proto2',
  serialized_pb=_b('\n\x18v3/asset/link/link.proto\x12\rv3.asset.link\"0\n\x11link_extra_header\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\t\"*\n\x0blink_cookie\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\t\"<\n\x0flink_form_input\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x02(\t\x12\x0c\n\x04type\x18\x03 \x02(\t\"L\n\tlink_form\x12.\n\x06inputs\x18\x01 \x03(\x0b\x32\x1e.v3.asset.link.link_form_input\x12\x0f\n\x07\x65nctype\x18\x02 \x02(\t\"M\n\x0flink_credential\x12\x0c\n\x04type\x18\x01 \x02(\t\x12\r\n\x05login\x18\x02 \x01(\t\x12\x10\n\x08password\x18\x03 \x01(\t\x12\x0b\n\x03url\x18\x04 \x01(\t\"\xb0\x02\n\x07Message\x12\x0b\n\x03url\x18\x01 \x02(\t\x12\x0e\n\x06method\x18\x02 \x02(\t\x12\x0e\n\x04\x62ody\x18\x03 \x01(\x0cH\x00\x12(\n\x04\x66orm\x18\x04 \x01(\x0b\x32\x18.v3.asset.link.link_formH\x00\x12\x37\n\rextra_headers\x18\x05 \x03(\x0b\x32 .v3.asset.link.link_extra_header\x12+\n\x07\x63ookies\x18\x06 \x03(\x0b\x32\x1a.v3.asset.link.link_cookie\x12\x32\n\ncredential\x18\x07 \x01(\x0b\x32\x1e.v3.asset.link.link_credential\x12&\n\x06parent\x18\x08 \x01(\x0b\x32\x16.v3.asset.link.MessageB\x0c\n\nbody_oneof')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_LINK_EXTRA_HEADER = _descriptor.Descriptor(
  name='link_extra_header',
  full_name='v3.asset.link.link_extra_header',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='v3.asset.link.link_extra_header.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='v3.asset.link.link_extra_header.value', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=43,
  serialized_end=91,
)


_LINK_COOKIE = _descriptor.Descriptor(
  name='link_cookie',
  full_name='v3.asset.link.link_cookie',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='v3.asset.link.link_cookie.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='v3.asset.link.link_cookie.value', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=93,
  serialized_end=135,
)


_LINK_FORM_INPUT = _descriptor.Descriptor(
  name='link_form_input',
  full_name='v3.asset.link.link_form_input',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='v3.asset.link.link_form_input.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='v3.asset.link.link_form_input.value', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type', full_name='v3.asset.link.link_form_input.type', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=137,
  serialized_end=197,
)


_LINK_FORM = _descriptor.Descriptor(
  name='link_form',
  full_name='v3.asset.link.link_form',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='inputs', full_name='v3.asset.link.link_form.inputs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='enctype', full_name='v3.asset.link.link_form.enctype', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=199,
  serialized_end=275,
)


_LINK_CREDENTIAL = _descriptor.Descriptor(
  name='link_credential',
  full_name='v3.asset.link.link_credential',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='v3.asset.link.link_credential.type', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='login', full_name='v3.asset.link.link_credential.login', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='password', full_name='v3.asset.link.link_credential.password', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='url', full_name='v3.asset.link.link_credential.url', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=277,
  serialized_end=354,
)


_MESSAGE = _descriptor.Descriptor(
  name='Message',
  full_name='v3.asset.link.Message',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='url', full_name='v3.asset.link.Message.url', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='method', full_name='v3.asset.link.Message.method', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='body', full_name='v3.asset.link.Message.body', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='form', full_name='v3.asset.link.Message.form', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='extra_headers', full_name='v3.asset.link.Message.extra_headers', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cookies', full_name='v3.asset.link.Message.cookies', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='credential', full_name='v3.asset.link.Message.credential', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='parent', full_name='v3.asset.link.Message.parent', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='body_oneof', full_name='v3.asset.link.Message.body_oneof',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=357,
  serialized_end=661,
)

_LINK_FORM.fields_by_name['inputs'].message_type = _LINK_FORM_INPUT
_MESSAGE.fields_by_name['form'].message_type = _LINK_FORM
_MESSAGE.fields_by_name['extra_headers'].message_type = _LINK_EXTRA_HEADER
_MESSAGE.fields_by_name['cookies'].message_type = _LINK_COOKIE
_MESSAGE.fields_by_name['credential'].message_type = _LINK_CREDENTIAL
_MESSAGE.fields_by_name['parent'].message_type = _MESSAGE
_MESSAGE.oneofs_by_name['body_oneof'].fields.append(
  _MESSAGE.fields_by_name['body'])
_MESSAGE.fields_by_name['body'].containing_oneof = _MESSAGE.oneofs_by_name['body_oneof']
_MESSAGE.oneofs_by_name['body_oneof'].fields.append(
  _MESSAGE.fields_by_name['form'])
_MESSAGE.fields_by_name['form'].containing_oneof = _MESSAGE.oneofs_by_name['body_oneof']
DESCRIPTOR.message_types_by_name['link_extra_header'] = _LINK_EXTRA_HEADER
DESCRIPTOR.message_types_by_name['link_cookie'] = _LINK_COOKIE
DESCRIPTOR.message_types_by_name['link_form_input'] = _LINK_FORM_INPUT
DESCRIPTOR.message_types_by_name['link_form'] = _LINK_FORM
DESCRIPTOR.message_types_by_name['link_credential'] = _LINK_CREDENTIAL
DESCRIPTOR.message_types_by_name['Message'] = _MESSAGE

link_extra_header = _reflection.GeneratedProtocolMessageType('link_extra_header', (_message.Message,), dict(
  DESCRIPTOR = _LINK_EXTRA_HEADER,
  __module__ = 'v3.asset.link.link_pb2'
  # @@protoc_insertion_point(class_scope:v3.asset.link.link_extra_header)
  ))
_sym_db.RegisterMessage(link_extra_header)

link_cookie = _reflection.GeneratedProtocolMessageType('link_cookie', (_message.Message,), dict(
  DESCRIPTOR = _LINK_COOKIE,
  __module__ = 'v3.asset.link.link_pb2'
  # @@protoc_insertion_point(class_scope:v3.asset.link.link_cookie)
  ))
_sym_db.RegisterMessage(link_cookie)

link_form_input = _reflection.GeneratedProtocolMessageType('link_form_input', (_message.Message,), dict(
  DESCRIPTOR = _LINK_FORM_INPUT,
  __module__ = 'v3.asset.link.link_pb2'
  # @@protoc_insertion_point(class_scope:v3.asset.link.link_form_input)
  ))
_sym_db.RegisterMessage(link_form_input)

link_form = _reflection.GeneratedProtocolMessageType('link_form', (_message.Message,), dict(
  DESCRIPTOR = _LINK_FORM,
  __module__ = 'v3.asset.link.link_pb2'
  # @@protoc_insertion_point(class_scope:v3.asset.link.link_form)
  ))
_sym_db.RegisterMessage(link_form)

link_credential = _reflection.GeneratedProtocolMessageType('link_credential', (_message.Message,), dict(
  DESCRIPTOR = _LINK_CREDENTIAL,
  __module__ = 'v3.asset.link.link_pb2'
  # @@protoc_insertion_point(class_scope:v3.asset.link.link_credential)
  ))
_sym_db.RegisterMessage(link_credential)

Message = _reflection.GeneratedProtocolMessageType('Message', (_message.Message,), dict(
  DESCRIPTOR = _MESSAGE,
  __module__ = 'v3.asset.link.link_pb2'
  # @@protoc_insertion_point(class_scope:v3.asset.link.Message)
  ))
_sym_db.RegisterMessage(Message)


# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cve.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name='cve.proto',
    package='',
    syntax='proto2',
    serialized_options=None,
    serialized_pb=_b(
        '\n\tcve.proto\"G\n\treference\x12\x0b\n\x03url\x18\x01 \x02(\t\x12\x0c\n\x04name\x18\x02 \x02(\t\x12\x11\n\trefsource\x18\x03 \x02(\t\x12\x0c\n\x04tags\x18\x04 \x03(\t\"W\n\rversion_match\x12\r\n\x05start\x18\x01 \x02(\t\x12\x15\n\rstart_include\x18\x02 \x02(\x08\x12\x0b\n\x03\x65nd\x18\x03 \x02(\t\x12\x13\n\x0b\x65nd_include\x18\x04 \x02(\x08\"\xdd\x01\n\tcpe_match\x12\x0c\n\x04part\x18\x01 \x02(\t\x12\x0e\n\x06vendor\x18\x02 \x01(\t\x12\x0f\n\x07product\x18\x03 \x01(\t\x12\x1f\n\x07version\x18\x04 \x01(\x0b\x32\x0e.version_match\x12\x0e\n\x06update\x18\x05 \x01(\t\x12\x0f\n\x07\x65\x64ition\x18\x06 \x01(\t\x12\x10\n\x08language\x18\x07 \x01(\t\x12\x12\n\nsw_edition\x18\x08 \x01(\t\x12\n\n\x02sw\x18\t \x01(\t\x12\n\n\x02hw\x18\n \x01(\t\x12\r\n\x05other\x18\x0b \x01(\t\x12\x12\n\nvulnerable\x18\x0c \x02(\x08\")\n\x06target\x12\x1f\n\x0b\x63pe_matches\x18\x01 \x03(\x0b\x32\n.cpe_match\"\xe1\x01\n\x03\x63ve\x12\x0f\n\x07scan_id\x18\x01 \x02(\x05\x12\x0e\n\x06\x63ve_id\x18\x02 \x02(\t\x12\x18\n\x07targets\x18\x03 \x03(\x0b\x32\x07.target\x12\x0b\n\x03\x63we\x18\x04 \x01(\x05\x12\x13\n\x0b\x64\x65scription\x18\x05 \x02(\t\x12\x1e\n\nreferences\x18\x06 \x03(\x0b\x32\n.reference\x12\x16\n\x0e\x63vss_v3_vector\x18\x07 \x01(\t\x12\x16\n\x0e\x63vss_v2_vector\x18\x08 \x01(\t\x12\x16\n\x0epublished_date\x18\t \x02(\t\x12\x15\n\rmodified_date\x18\x13 \x02(\t')
)

_REFERENCE = _descriptor.Descriptor(
    name='reference',
    full_name='reference',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='url', full_name='reference.url', index=0,
            number=1, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='name', full_name='reference.name', index=1,
            number=2, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='refsource', full_name='reference.refsource', index=2,
            number=3, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='tags', full_name='reference.tags', index=3,
            number=4, type=9, cpp_type=9, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto2',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=13,
    serialized_end=84,
)

_VERSION_MATCH = _descriptor.Descriptor(
    name='version_match',
    full_name='version_match',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='start', full_name='version_match.start', index=0,
            number=1, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='start_include', full_name='version_match.start_include', index=1,
            number=2, type=8, cpp_type=7, label=2,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='end', full_name='version_match.end', index=2,
            number=3, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='end_include', full_name='version_match.end_include', index=3,
            number=4, type=8, cpp_type=7, label=2,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto2',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=86,
    serialized_end=173,
)

_CPE_MATCH = _descriptor.Descriptor(
    name='cpe_match',
    full_name='cpe_match',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='part', full_name='cpe_match.part', index=0,
            number=1, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='vendor', full_name='cpe_match.vendor', index=1,
            number=2, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='product', full_name='cpe_match.product', index=2,
            number=3, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='version', full_name='cpe_match.version', index=3,
            number=4, type=11, cpp_type=10, label=1,
            has_default_value=False, default_value=None,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='update', full_name='cpe_match.update', index=4,
            number=5, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='edition', full_name='cpe_match.edition', index=5,
            number=6, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='language', full_name='cpe_match.language', index=6,
            number=7, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='sw_edition', full_name='cpe_match.sw_edition', index=7,
            number=8, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='sw', full_name='cpe_match.sw', index=8,
            number=9, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='hw', full_name='cpe_match.hw', index=9,
            number=10, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='other', full_name='cpe_match.other', index=10,
            number=11, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='vulnerable', full_name='cpe_match.vulnerable', index=11,
            number=12, type=8, cpp_type=7, label=2,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto2',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=176,
    serialized_end=397,
)

_TARGET = _descriptor.Descriptor(
    name='target',
    full_name='target',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='cpe_matches', full_name='target.cpe_matches', index=0,
            number=1, type=11, cpp_type=10, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto2',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=399,
    serialized_end=440,
)

_CVE = _descriptor.Descriptor(
    name='cve',
    full_name='cve',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='scan_id', full_name='cve.scan_id', index=0,
            number=1, type=5, cpp_type=1, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='cve_id', full_name='cve.cve_id', index=1,
            number=2, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='targets', full_name='cve.targets', index=2,
            number=3, type=11, cpp_type=10, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='cwe', full_name='cve.cwe', index=3,
            number=4, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='description', full_name='cve.description', index=4,
            number=5, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='references', full_name='cve.references', index=5,
            number=6, type=11, cpp_type=10, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='cvss_v3_vector', full_name='cve.cvss_v3_vector', index=6,
            number=7, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='cvss_v2_vector', full_name='cve.cvss_v2_vector', index=7,
            number=8, type=9, cpp_type=9, label=1,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='published_date', full_name='cve.published_date', index=8,
            number=9, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='modified_date', full_name='cve.modified_date', index=9,
            number=19, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto2',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=443,
    serialized_end=668,
)

_CPE_MATCH.fields_by_name['version'].message_type = _VERSION_MATCH
_TARGET.fields_by_name['cpe_matches'].message_type = _CPE_MATCH
_CVE.fields_by_name['targets'].message_type = _TARGET
_CVE.fields_by_name['references'].message_type = _REFERENCE
DESCRIPTOR.message_types_by_name['reference'] = _REFERENCE
DESCRIPTOR.message_types_by_name['version_match'] = _VERSION_MATCH
DESCRIPTOR.message_types_by_name['cpe_match'] = _CPE_MATCH
DESCRIPTOR.message_types_by_name['target'] = _TARGET
DESCRIPTOR.message_types_by_name['cve'] = _CVE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

reference = _reflection.GeneratedProtocolMessageType('reference', (_message.Message,), dict(
    DESCRIPTOR=_REFERENCE,
    __module__='cve_pb2'
    # @@protoc_insertion_point(class_scope:reference)
))
_sym_db.RegisterMessage(reference)

version_match = _reflection.GeneratedProtocolMessageType('version_match', (_message.Message,), dict(
    DESCRIPTOR=_VERSION_MATCH,
    __module__='cve_pb2'
    # @@protoc_insertion_point(class_scope:version_match)
))
_sym_db.RegisterMessage(version_match)

cpe_match = _reflection.GeneratedProtocolMessageType('cpe_match', (_message.Message,), dict(
    DESCRIPTOR=_CPE_MATCH,
    __module__='cve_pb2'
    # @@protoc_insertion_point(class_scope:cpe_match)
))
_sym_db.RegisterMessage(cpe_match)

target = _reflection.GeneratedProtocolMessageType('target', (_message.Message,), dict(
    DESCRIPTOR=_TARGET,
    __module__='cve_pb2'
    # @@protoc_insertion_point(class_scope:target)
))
_sym_db.RegisterMessage(target)

cve = _reflection.GeneratedProtocolMessageType('cve', (_message.Message,), dict(
    DESCRIPTOR=_CVE,
    __module__='cve_pb2'
    # @@protoc_insertion_point(class_scope:cve)
))
_sym_db.RegisterMessage(cve)

# @@protoc_insertion_point(module_scope)



"""Common logic for Mobile Assets of type .APK package file AAB package file and IPA .
This module takes care of preparing the application file and calling the create mobile scan API.
"""
import io
import click

from ostorlab.cli.ci_scan.run import run
from ostorlab.apis.runners import authenticated_runner
from ostorlab.apis.runners import runner as base_runner
from ostorlab.apis import scan_create as scan_create_api


def run_mobile_scan(ctx: click.core.Context, file: io.FileIO, asset_type: scan_create_api.MobileAssetType) -> None:
    """Create scan for mobile application package file."""
    ci_logger = ctx.obj['ci_logger']
    if ctx.obj.get('api_key'):
        scan_profile = ctx.obj['scan_profile']
        title = ctx.obj['title']
        break_on_risk_rating = ctx.obj['break_on_risk_rating']
        max_wait_minutes = ctx.obj['max_wait_minutes']
        runner = authenticated_runner.AuthenticatedAPIRunner(api_key=ctx.obj.get('api_key'))
        try:
            scan_result = runner.execute(
                scan_create_api.CreateMobileScanAPIRequest(title=title,
                                                           asset_type=asset_type,
                                                           scan_profile=scan_profile,
                                                           application=file))
            scan_id = scan_result.get('data').get('createMobileScan').get('scan').get('id')
            ci_logger.output(name='scan_id', value=scan_id)
            ci_logger.info(f'Scan created with id {scan_id}.')
            if break_on_risk_rating is not None:
                run.apply_break_scan_risk_rating(break_on_risk_rating, scan_id, max_wait_minutes, runner, ci_logger)

        except base_runner.ResponseError as e:
            ci_logger.error(f'Could not start the scan. {e}')
            raise click.exceptions.Exit(2) from e
        except TimeoutError:
            ci_logger.error('The scan is still running.')
            raise click.exceptions.Exit(2) from None
    else:
        ci_logger.error('API key not not provided.')
        raise click.exceptions.Exit(2) from None



"""Asset of type Link."""
import logging

import click

from ostorlab.assets import link as link_asset
from ostorlab.cli.scan.run import run

logger = logging.getLogger(__name__)


@run.run.command()
@click.option('--url', help='Url to scan.', required=True)
@click.option('--method', help='HTTP method.', required=True)
@click.pass_context
def link(ctx: click.core.Context, url: str, method: str) -> None:
    """Run scan for link."""
    runtime = ctx.obj['runtime']
    asset = link_asset.Link(url=url, method=method)
    logger.debug('scanning asset %s', asset)
    runtime.scan(title=ctx.obj['title'], agent_group_definition=ctx.obj['agent_group_definition'], assets=[asset])



"""Lite local runtime runs agents locally with limited setup, like no MQ service and no defaults agents.

The local runtime requires Docker Swarm to run robust long-running services with a set of configured services.
"""
import logging
from typing import List
from typing import Optional

import click
import docker
import tenacity
from docker.models import services as docker_models_services

from ostorlab import exceptions
from ostorlab.assets import asset as base_asset
from ostorlab.cli import console as cli_console, dumpers
from ostorlab.cli import docker_requirements_checker
from ostorlab.cli import install_agent
from ostorlab.runtimes import definitions
from ostorlab.runtimes import runtime
from ostorlab.runtimes.lite_local import agent_runtime
from ostorlab.utils import volumes

NETWORK_PREFIX = 'ostorlab_lite_local_network'

logger = logging.getLogger(__name__)
console = cli_console.Console()

ASSET_INJECTION_AGENT_DEFAULT = 'agent/ostorlab/inject_asset'


class UnhealthyService(exceptions.OstorlabError):
    """A service by the runtime is considered unhealthy."""


class AgentNotInstalled(exceptions.OstorlabError):
    """Agent image not installed."""


class AgentNotHealthy(exceptions.OstorlabError):
    """Agent not healthy."""


def _has_container_image(agent: definitions.AgentSettings):
    """Check if container image is available"""
    return agent.container_image is not None


def _is_service_type_run(service: docker_models_services.Service) -> bool:
    """Checks if the service should run once or should be continuously running.

    Args:
        service: Docker service.

    Returns:
        Bool indicating if the service is run-once or long-running.
    """
    return service.attrs['Spec']['TaskTemplate']['RestartPolicy']['Condition'] == 'none'


class LiteLocalRuntime(runtime.Runtime):
    """Lite Local runtime runs agents locally using Docker Swarm.
    Lite Local runtime starts all the agents listed in the `AgentRunDefinition`, and then injects the target asset.
    """

    def __init__(self, scan_id: str, bus_url: str, bus_vhost: str, bus_management_url: str,
                 bus_exchange_topic: str, network: str, redis_url: str) -> None:
        """Set runtime attributes.

        Args:
            scan_id: Provided scan identifier, will be used to define the runtime name.
            bus_url: Bus URL, may contain credentials.
            bus_vhost: Bus virtual host, common default is / but none is provided here.
            bus_management_url: Bus management URL, typically runs on a separate port over https.
            bus_exchange_topic: Bus exchange topic.
            network: Docker network name to attach to.
            redis_url: Redis URL.
        """
        super().__init__()

        if not all([scan_id, bus_url, bus_vhost, bus_management_url, bus_exchange_topic, network, redis_url]):
            raise ValueError('Missing required fields.')

        self.scan_id = scan_id
        self._bus_url = bus_url
        self._bus_vhost = bus_vhost
        self._bus_management_url = bus_management_url
        self._bus_exchange_topic = bus_exchange_topic
        self._network = network
        self._redis_url = redis_url

        if not docker_requirements_checker.is_docker_installed():
            console.error('Docker is not installed.')
            raise click.exceptions.Exit(2)
        elif not docker_requirements_checker.is_user_permitted():
            console.error('User does not have permissions to run docker.')
            raise click.exceptions.Exit(2)
        elif not docker_requirements_checker.is_docker_working():
            console.error('Error using docker.')
            raise click.exceptions.Exit(2)
        else:
            if not docker_requirements_checker.is_swarm_initialized():
                docker_requirements_checker.init_swarm()
            self._docker_client = docker.from_env()

    @property
    def name(self) -> str:
        """Lite Local runtime instance name."""
        return self.scan_id

    @property
    def network(self) -> str:
        """Lite Local runtime network name.

        Returns:
            Lite Local runtime network name.
        """
        return self._network

    def can_run(self, agent_group_definition: definitions.AgentGroupDefinition) -> bool:
        """Checks if the runtime can run the provided agent run definition.

        Args:
            agent_group_definition: Agent and Agent group definition.

        Returns:
            Always true for the moment as the lite local runtime doesn't have restrictions on what it can run.
        """
        del agent_group_definition
        return True

    def scan(self, title: str, agent_group_definition: definitions.AgentGroupDefinition,
             assets: List[base_asset.Asset]) -> None:
        """Start scan on asset using the provided agent run definition.

        The scan takes care of starting all the scan required services, ensuring they are healthy, starting all the
         agents, ensuring they are healthy and then injects the target asset.

        Args:
            title: Scan title
            agent_group_definition: Agent run definition defines the set of agents and how agents are configured.
            assets: the target asset to scan.

        Returns:
            None
        """
        try:
            console.info('Starting agents')
            self._start_agents(agent_group_definition)
            console.info('Checking agents are healthy')
            is_healthy = self._check_agents_healthy()
            if is_healthy is False:
                raise AgentNotHealthy()
            console.info('Injecting assets')
            self._inject_assets(assets=assets)
        except AgentNotHealthy:
            console.error('Agent not starting')
            self.stop(self.scan_id)
        except AgentNotInstalled as e:
            console.error(f'Agent {e} not installed')
            self.stop(self.scan_id)
        except agent_runtime.MissingAgentDefinitionLabel as e:
            console.error(f'Missing agent definition {e}. This is probably due to building the image directly with'
                          f' docker instead of `ostorlab agent build` command')
            self.stop(self.scan_id)

    def stop(self, scan_id: str) -> None:
        """Remove a service (scan) belonging to universe with scan_id(Universe Id).

        Args:
            scan_id: The id of the scan to stop.
        """

        stopped_services = []
        stopped_network = []
        stopped_configs = []
        client = docker.from_env()
        services = client.services.list()
        for service in services:
            service_labels = service.attrs['Spec']['Labels']
            logger.debug('comparing %s and %s', service_labels.get(
                'ostorlab.universe'), scan_id)
            if service_labels.get('ostorlab.universe') == scan_id:
                stopped_services.append(service)
                service.remove()

        networks = client.networks.list()
        for network in networks:
            network_labels = network.attrs['Labels']
            if network_labels is not None and network_labels.get('ostorlab.universe') == scan_id:
                logger.debug('removing network %s', network_labels)
                stopped_network.append(network)
                network.remove()

        configs = client.configs.list()
        for config in configs:
            config_labels = config.attrs['Spec']['Labels']
            if config_labels.get('ostorlab.universe') == scan_id:
                logger.debug('removing config %s', config_labels)
                stopped_configs.append(config)
                config.remove()

        if stopped_services or stopped_network or stopped_configs:
            console.success('All scan components stopped.')

    def _check_agents_healthy(self):
        """Checks if an agent is healthy."""
        return self._are_agents_ready()

    def _start_agents(self, agent_group_definition: definitions.AgentGroupDefinition):
        """Starts all the agents as list in the agent run definition."""
        for agent in agent_group_definition.agents:
            self._start_agent(agent, extra_configs=[])

    def _start_agent(self, agent: definitions.AgentSettings,
                     extra_configs: Optional[List[docker.types.ConfigReference]] = None,
                     extra_mounts: Optional[List[docker.types.Mount]] = None
                     ) -> None:
        """Start agent based on provided definition.

        Args:
            agent: An agent definition containing all the settings of how agent should run and what arguments to pass.
        """
        logger.info('starting agent %s with %s', agent.key, agent.args)

        if _has_container_image(agent) is False:
            raise AgentNotInstalled(agent.key)

        runtime_agent = agent_runtime.AgentRuntime(agent,
                                                   self.name,
                                                   self._docker_client,
                                                   self._bus_url,
                                                   self._bus_vhost,
                                                   self._bus_management_url,
                                                   self._bus_exchange_topic,
                                                   self._redis_url
                                                   )
        agent_service = runtime_agent.create_agent_service(self.network, extra_configs, extra_mounts)

        if agent.replicas > 1:
            # Ensure the agent service had to
            # TODO(alaeddine): Check if sleep if really needed and if it is, implement a parallel way to start agents
            #  and scale them.
            # time.sleep(10)
            self._scale_service(agent_service, agent.replicas)

    @tenacity.retry(stop=tenacity.stop_after_attempt(20),
                    wait=tenacity.wait_exponential(multiplier=1, max=12),
                    # return last value and don't raise RetryError exception.
                    retry_error_callback=lambda lv: lv.outcome.result(),
                    retry=tenacity.retry_if_result(lambda v: v is False))
    def _is_service_healthy(self, service: docker_models_services.Service, replicas=None) -> bool:
        """Checks if a docker service is healthy by checking all tasks status."""
        logger.debug('checking Spec service %s', service.name)
        try:
            if not replicas:
                replicas = service.attrs['Spec']['Mode']['Replicated']['Replicas']
            return replicas == len([task for task in service.tasks() if task['Status']['State'] == 'running'])
        except docker.errors.NotFound:
            return False

    def _list_agent_services(self):
        """List the services of type agents. All agent service must start with agent_."""
        services = self._docker_client.services.list(
            filters={'label': f'ostorlab.universe={self.name}'})
        for service in services:
            if service.name.startswith('agent_'):
                yield service

    def _inject_assets(self, assets: List[base_asset.Asset]):
        """Injects the scan target assets."""

        contents = {}
        for i, asset in enumerate(assets):
            contents[f'asset.binproto_{i}'] = asset.to_proto()
            contents[f'selector.txt_{i}'] = asset.selector.encode()

        volumes.create_volume(f'asset_{self.name}', contents)

        inject_asset_agent_settings = definitions.AgentSettings(key=ASSET_INJECTION_AGENT_DEFAULT,
                                                                restart_policy='none')
        self._start_agent(agent=inject_asset_agent_settings,
                          extra_mounts=[
                              docker.types.Mount(
                                  target='/asset', source=f'asset_{self.name}', type='volume'
                              )])

    def _scale_service(self, service: docker_models_services.Service, replicas: int) -> None:
        """Calling scale directly on the service causes an API error. This is a workaround that simulates refreshing
         the service object, then calling the scale API."""
        for s in self._docker_client.services.list():
            if s.name == service.name:
                s.scale(replicas)

    @tenacity.retry(stop=tenacity.stop_after_attempt(20),
                    wait=tenacity.wait_exponential(multiplier=1, max=20),
                    # return last value and don't raise RetryError exception.
                    retry_error_callback=lambda lv: lv.outcome.result(),
                    retry=tenacity.retry_if_result(lambda v: v is False))
    def list(self, **kwargs):
        raise NotImplementedError()

    def _are_agents_ready(self, fail_fast=True) -> bool:
        """Checks that all agents are ready and healthy while taking into account the run type of agent
         (once vs long-running)."""
        logger.info('listing services ...')
        agent_services = list(self._list_agent_services())
        for service in agent_services:
            logger.info('checking %s ...', service.name)
            if not _is_service_type_run(service):
                if self._is_service_healthy(service):
                    logger.info('agent service %s is healthy', service.name)
                else:
                    logger.error('agent service %s is not healthy', service.name)
                    if fail_fast:
                        return False
        return True

    def install(self) -> None:
        """Installs the default agents.

        Returns:
            None
        """
        install_agent.install(agent_key=ASSET_INJECTION_AGENT_DEFAULT)
    def dump_vulnz(self, scan_id: int, dumper: dumpers.VulnzDumper):
        """Dump vulnerabilities to a file in a specific format.
            Returns:
            None
        """
        pass



"""mock agent implements the required methods to test the agent's behavior without using external components."""
import pytest

from typing import List

from ostorlab.agent import message as msg


@pytest.fixture(scope='function')
def agent_mock(mocker):
    """This fixtures patches all the Agent components and returns the list of messages emitted """
    emitted_messages: List[msg.Message] = []
    mocker.patch('ostorlab.agent.mixins.agent_mq_mixin.AgentMQMixin.mq_init', return_value=None)
    mocker.patch('ostorlab.agent.mixins.agent_mq_mixin.AgentMQMixin.mq_run', return_value=None)
    mocker.patch(
        'ostorlab.agent.mixins.agent_healthcheck_mixin.AgentHealthcheckMixin.start_healthcheck',
        return_value=None
    )
    mocker.patch(
        'ostorlab.agent.mixins.agent_healthcheck_mixin.AgentHealthcheckMixin.add_healthcheck',
        return_value=None
    )
    mocker.patch(
        'ostorlab.agent.mixins.agent_healthcheck_mixin.AgentHealthcheckMixin.__init__',
        return_value=None
    )

    def mq_send_message(key, message):
        # we need to remove the last part of the key f'{selector}.{uuid.uuid1()}'
        agent_message = msg.Message.from_raw('.'.join(key.split('.')[:-1]), message)
        emitted_messages.append(agent_message)

    mocker.patch('ostorlab.agent.mixins.agent_mq_mixin.AgentMQMixin.mq_send_message', side_effect=mq_send_message)
    yield emitted_messages
    emitted_messages = []


@pytest.fixture(scope='function')
def agent_persist_mock(mocker):
    """This fixtures patches the Agent persist component and returns the list storage state"""
    storage = {}

    def _set_is_member(key, value):
        """Check values are present in the storage dict."""
        return key in storage and value in storage[key]

    def _set_add(key, value):
        """Add members to the storage dict and emulate return value."""
        if _set_is_member(key, value):
            return False
        else:
            storage.setdefault(key, set()).add(value)
            return True

    def _set_len(key):
        if key in storage:
            return len(storage[key])

    def _set_members(key):
        if key in storage:
            return storage[key]

    def _get(key):
        if key in storage:
            return storage[key]

    def _add(key, value):
        """Check values are present in the storage dict."""
        storage[key] = value

    mocker.patch('ostorlab.agent.mixins.agent_persist_mixin.AgentPersistMixin.set_add',
                 side_effect=_set_add)
    mocker.patch('ostorlab.agent.mixins.agent_persist_mixin.AgentPersistMixin.set_is_member',
                 side_effect=_set_is_member)
    mocker.patch('ostorlab.agent.mixins.agent_persist_mixin.AgentPersistMixin.set_len',
                 side_effect=_set_len)
    mocker.patch('ostorlab.agent.mixins.agent_persist_mixin.AgentPersistMixin.set_members',
                 side_effect=_set_members)
    mocker.patch('ostorlab.agent.mixins.agent_persist_mixin.AgentPersistMixin.get',
                 side_effect=_get)
    mocker.patch('ostorlab.agent.mixins.agent_persist_mixin.AgentPersistMixin.add',
                 side_effect=_add)
    yield storage
    storage = {}









# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v3/capture/request_response/request_response.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='v3/capture/request_response/request_response.proto',
  package='v3.capture.request_response',
  syntax='proto2',
  serialized_pb=_b('\n2v3/capture/request_response/request_response.proto\x12\x1bv3.capture.request_response\"%\n\x06header\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\t\"j\n\x07request\x12\x0e\n\x06method\x18\x01 \x02(\t\x12\x0b\n\x03url\x18\x02 \x02(\t\x12\x34\n\x07headers\x18\x03 \x03(\x0b\x32#.v3.capture.request_response.header\x12\x0c\n\x04\x62ody\x18\x04 \x01(\x0c\"\xba\x01\n\x08response\x12\x0b\n\x03url\x18\x01 \x02(\t\x12\x0e\n\x06status\x18\x02 \x02(\x05\x12\x13\n\x0bstatus_text\x18\x03 \x02(\t\x12\x34\n\x07headers\x18\x04 \x03(\x0b\x32#.v3.capture.request_response.header\x12\x11\n\tremote_ip\x18\x05 \x01(\t\x12\x13\n\x0bremote_port\x18\x06 \x01(\x05\x12\x10\n\x08protocol\x18\x07 \x01(\t\x12\x0c\n\x04\x62ody\x18\x08 \x01(\x0c\"y\n\x07Message\x12\x35\n\x07request\x18\x01 \x02(\x0b\x32$.v3.capture.request_response.request\x12\x37\n\x08response\x18\x02 \x01(\x0b\x32%.v3.capture.request_response.response')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_HEADER = _descriptor.Descriptor(
  name='header',
  full_name='v3.capture.request_response.header',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='v3.capture.request_response.header.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='v3.capture.request_response.header.value', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=83,
  serialized_end=120,
)


_REQUEST = _descriptor.Descriptor(
  name='request',
  full_name='v3.capture.request_response.request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='method', full_name='v3.capture.request_response.request.method', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='url', full_name='v3.capture.request_response.request.url', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='headers', full_name='v3.capture.request_response.request.headers', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='body', full_name='v3.capture.request_response.request.body', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=122,
  serialized_end=228,
)


_RESPONSE = _descriptor.Descriptor(
  name='response',
  full_name='v3.capture.request_response.response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='url', full_name='v3.capture.request_response.response.url', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='status', full_name='v3.capture.request_response.response.status', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='status_text', full_name='v3.capture.request_response.response.status_text', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='headers', full_name='v3.capture.request_response.response.headers', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='remote_ip', full_name='v3.capture.request_response.response.remote_ip', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='remote_port', full_name='v3.capture.request_response.response.remote_port', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='protocol', full_name='v3.capture.request_response.response.protocol', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='body', full_name='v3.capture.request_response.response.body', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=231,
  serialized_end=417,
)


_MESSAGE = _descriptor.Descriptor(
  name='Message',
  full_name='v3.capture.request_response.Message',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='request', full_name='v3.capture.request_response.Message.request', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='response', full_name='v3.capture.request_response.Message.response', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=419,
  serialized_end=540,
)

_REQUEST.fields_by_name['headers'].message_type = _HEADER
_RESPONSE.fields_by_name['headers'].message_type = _HEADER
_MESSAGE.fields_by_name['request'].message_type = _REQUEST
_MESSAGE.fields_by_name['response'].message_type = _RESPONSE
DESCRIPTOR.message_types_by_name['header'] = _HEADER
DESCRIPTOR.message_types_by_name['request'] = _REQUEST
DESCRIPTOR.message_types_by_name['response'] = _RESPONSE
DESCRIPTOR.message_types_by_name['Message'] = _MESSAGE

header = _reflection.GeneratedProtocolMessageType('header', (_message.Message,), dict(
  DESCRIPTOR = _HEADER,
  __module__ = 'v3.capture.request_response.request_response_pb2'
  # @@protoc_insertion_point(class_scope:v3.capture.request_response.header)
  ))
_sym_db.RegisterMessage(header)

request = _reflection.GeneratedProtocolMessageType('request', (_message.Message,), dict(
  DESCRIPTOR = _REQUEST,
  __module__ = 'v3.capture.request_response.request_response_pb2'
  # @@protoc_insertion_point(class_scope:v3.capture.request_response.request)
  ))
_sym_db.RegisterMessage(request)

response = _reflection.GeneratedProtocolMessageType('response', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSE,
  __module__ = 'v3.capture.request_response.request_response_pb2'
  # @@protoc_insertion_point(class_scope:v3.capture.request_response.response)
  ))
_sym_db.RegisterMessage(response)

Message = _reflection.GeneratedProtocolMessageType('Message', (_message.Message,), dict(
  DESCRIPTOR = _MESSAGE,
  __module__ = 'v3.capture.request_response.request_response_pb2'
  # @@protoc_insertion_point(class_scope:v3.capture.request_response.Message)
  ))
_sym_db.RegisterMessage(Message)


# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: coverage.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name='coverage.proto',
    package='',
    syntax='proto2',
    serialized_options=None,
    serialized_pb=_b(
        '\n\x0e\x63overage.proto\"8\n\x10payload_validity\x12\x12\n\npayload_id\x18\x01 \x01(\x05\x12\x10\n\x08validity\x18\x02 \x02(\x08\"\x92\x01\n\x08\x63overage\x12\x0f\n\x07scan_id\x18\x01 \x02(\x05\x12\x15\n\rmodel_version\x18\x02 \x02(\x05\x12\x34\n\x19previous_payload_validity\x18\x03 \x03(\x0b\x32\x11.payload_validity\x12\x10\n\x08test_ids\x18\x04 \x03(\x05\x12\x16\n\x0eparent_node_id\x18\x05 \x01(\x05')
)

_PAYLOAD_VALIDITY = _descriptor.Descriptor(
    name='payload_validity',
    full_name='payload_validity',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='payload_id', full_name='payload_validity.payload_id', index=0,
            number=1, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='validity', full_name='payload_validity.validity', index=1,
            number=2, type=8, cpp_type=7, label=2,
            has_default_value=False, default_value=False,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto2',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=18,
    serialized_end=74,
)

_COVERAGE = _descriptor.Descriptor(
    name='coverage',
    full_name='coverage',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='scan_id', full_name='coverage.scan_id', index=0,
            number=1, type=5, cpp_type=1, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='model_version', full_name='coverage.model_version', index=1,
            number=2, type=5, cpp_type=1, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='previous_payload_validity', full_name='coverage.previous_payload_validity', index=2,
            number=3, type=11, cpp_type=10, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='test_ids', full_name='coverage.test_ids', index=3,
            number=4, type=5, cpp_type=1, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
        _descriptor.FieldDescriptor(
            name='parent_node_id', full_name='coverage.parent_node_id', index=4,
            number=5, type=5, cpp_type=1, label=1,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            serialized_options=None, file=DESCRIPTOR),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto2',
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=77,
    serialized_end=223,
)

_COVERAGE.fields_by_name['previous_payload_validity'].message_type = _PAYLOAD_VALIDITY
DESCRIPTOR.message_types_by_name['payload_validity'] = _PAYLOAD_VALIDITY
DESCRIPTOR.message_types_by_name['coverage'] = _COVERAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

payload_validity = _reflection.GeneratedProtocolMessageType('payload_validity', (_message.Message,), dict(
    DESCRIPTOR=_PAYLOAD_VALIDITY,
    __module__='coverage_pb2'
    # @@protoc_insertion_point(class_scope:payload_validity)
))
_sym_db.RegisterMessage(payload_validity)

coverage = _reflection.GeneratedProtocolMessageType('coverage', (_message.Message,), dict(
    DESCRIPTOR=_COVERAGE,
    __module__='coverage_pb2'
    # @@protoc_insertion_point(class_scope:coverage)
))
_sym_db.RegisterMessage(coverage)

# @@protoc_insertion_point(module_scope)



"""Mixin to enable distributed storage for a group of agent replicas or for a single agent that needs reliable storage.

A typical use-case is ensuring that a message value is processed only once in case multiple other agent are producing
duplicates or similar ones.

Typical usage:

```python
    status_agent = agent_persist_mixin.AgentPersistMixin(agent_settings)
    status_agent.set_add('crawler_agent_asset_dna', dna)
    is_new = not status_agent.set_is_member()
```
"""
import redis

from ostorlab.runtimes import definitions as runtime_definitions


class AgentPersistMixin:
    """Agent mixin to persist data. The mixin enables distributed storage of a group of agent replicas or for a single
     agent that needs reliable storage."""

    def __init__(self, agent_settings: runtime_definitions.AgentSettings) -> None:
        """Initializes the mixin from the agent settings.

        Args:
            agent_settings: Agent runtime settings.
        """
        if agent_settings.redis_url is None:
            raise ValueError('agent settings is missing redis url')
        self._redis_client = redis.Redis.from_url(agent_settings.redis_url)

    def set_add(self, key, value) -> bool:
        """Helper function that takes care of reporting if the specified DNA has been tested in the past, or mark it
        as tested.
        The method can be used to sync multiple agents that may encounter the same test input but need to test it
        only once.
        Storage is shared between agents, ensure the keys used are unique.

        Args:
            key: Set key.
            value: List of values to add to set.

        Returns:
            True if it is a new member, False otherwise.
        """
        return bool(self._redis_client.sadd(key, value))

    def set_is_member(self, key, value) -> bool:
        """Indicates whether value is member of the set identified by key.

        Args:
            key: The set key.
            value: The value to check.

        Returns:
            True if it is a member, False otherwise.
        """
        return self._redis_client.sismember(key, value)

    def set_len(self, key) -> bool:
        """Helper function that returns the set cardinality (number of elements) of the set stored at key.
        The method can be used to sync multiple agents that may receive test inputs but need to test
        less than X test inputs.
        Storage is shared between agents, ensure the keys used are unique.

        Args:
            key: Set key.

        Returns:
            the cardinality (number of elements) of the set, or 0 if key does not exist.
        """
        return self._redis_client.scard(key)

    def set_members(self, key) -> bool:
        """Helper function that returns the value of key.

        Args:
            key: Set key.

        Returns:
             the value of key, or nil when key does not exist.
        """
        return self._redis_client.smembers(key)

    def add(self, key, value) -> bool:
        """Helper function that Set key to hold the string value.

        Args:
            key: key.
            value: String.

        Returns:
            String status of the set command.
        """
        return self._redis_client.set(key, value)

    def get(self, key) -> bool:
        """Get the value of key. If the key does not exist the special value nil is returned.

        Args:
            key: key.

        Returns:
            the value of key, or nil when key does not exist.
        """
        return self._redis_client.get(key)



"""Abstract Asset class to define the scan target and its properties."""
import abc
from ostorlab import exceptions
from ostorlab.agent.message import serializer

class MissingTargetSelector(exceptions.OstorlabError):
    """Missing asset selector definition."""


class Asset(abc.ABC):
    """Abstract Asset class to define the scan target and its properties."""

    selector: str = None

    def to_proto(self) -> bytes:
        if self.selector is None:
            raise MissingTargetSelector()
        return serializer.serialize(self.selector, self.__dict__).SerializeToString()


def selector(target: str):
    """Decorator to define an asset selector for serialization.

    Args:
        target: Target selector.

    Returns:
        Selector initializer function.
    """

    def _selector_initializer(asset: Asset) -> Asset:
        """Set asset selector."""
        asset.selector = target
        return asset

    return _selector_initializer



"""Asset of type Domain Name."""
import logging
from typing import List

import click

from ostorlab.assets import domain_name
from ostorlab.cli.scan.run import run

logger = logging.getLogger(__name__)


@run.run.command(name='domain-name')
@click.argument('names', required=False, nargs=-1)
@click.pass_context
def domain_name_cli(ctx: click.core.Context, names: List[str]) -> None:
    """Run scan for Domain Name asset."""
    runtime = ctx.obj['runtime']
    assets = []
    for d in names:
        assets.append(domain_name.DomainName(name=d))
    logger.debug('scanning assets %s', [str(asset) for asset in assets])
    runtime.scan(title=ctx.obj['title'], agent_group_definition=ctx.obj['agent_group_definition'], assets=assets)
